#include "dispatcher.h"

rfbBool Dispatcher::MallocFrameBuffer(rfbClient *client)
{
    Dispatcher *dispatcher = (Dispatcher *) rfbClientGetClientData(client, 0);
    emit dispatcher->resizeFramebuffer(QSize(client->width, client->height));
    return TRUE;
}

void Dispatcher::GotCopyRect(rfbClient *client, int src_x, int src_y, int w, int h, int dest_x, int dest_y)
{
    Dispatcher *dispatcher = (Dispatcher *) rfbClientGetClientData(client, 0);
    dispatcher->uploader().gotCopyRect(src_x, src_y, w, h, dest_x, dest_y);
}

void Dispatcher::GotFillRect(rfbClient *client, int x, int y, int w, int h, uint32_t colour)
{
    Dispatcher *dispatcher = (Dispatcher *) rfbClientGetClientData(client, 0);
    int red = (colour >> m_client->format.redShift) & m_client->format.redMax;
    int green = (colour >> m_client->format.greenShift) & m_client->format.greenMax;
    int blue = (colour >> m_client->format.blueShift) & m_client->format.blueMax;
    dispatcher->uploader().gotFillRect(x, y, w, h, red, green, blue);
}

void Dispatcher::GotBitmap(rfbClient *client, const uint8_t *buffer, int x, int y, int w, int h)
{
    QImage img;
    Dispatcher *dispatcher = (Dispatcher *) rfbClientGetClientData(client, 0);
    switch(dispatcher->bitsPerPixel()) {
    case 8:
        img = QImage(buffer, w, h, QImage::Format_Grayscale8);
        break;
    case 16:
        img = QImage(buffer, w, h, QImage::Format_RGB16);
        break;
    case 32:
        img = QImage(buffer, w, h, QImage::Format_RGB32);
        break;
    default:
        emit dispatcher->error();
        return;
    }
    // We have to make a deep copy because libvnc might free the buffer as soon as this function returns
    dispatcher->uploader().gotBitmap(img.copy(), x, y, w, h);
}

rfbBool Dispatcher::GotJpeg(rfbClient *client, const uint8_t *buffer, int length, int x, int y, int w, int h)
{
    Dispatcher *dispatcher = (Dispatcher *) rfbClientGetClientData(client, 0);

    QByteArray jpeg;
    jpeg.setRawData(buffer, length);

    JpegDecoder *decoder = new JpegDecoder(jpeg, position, size);
    connect(decoder, &JpegDecoder::finished, dispatcher, &Dispatcher::decodingFinished);
    QThreadPool::globalInstance()->start(decoder);
    dispatcher->incrementDecoderCount();

    return TRUE;
}

void Dispatcher::FinishedFrameBufferUpdate(rfbClient *client)
{
    Dispatcher *dispatcher = (Dispatcher *) rfbClientGetClientData(client, 0);
    dispatcher->setFinished(true);
    if(decoderCount() == 0) {
        dispatcher->uploader().finishedUpdate();
        emit dispatcher->updateFramebuffer();
    }
}

unsigned int Dispatcher::decoderCount() const
{
    return m_decoderCount;
}

unsigned int Dispatcher::incrementDecoderCount()
{
    return ++m_decoderCount;
}

unsigned int Dispatcher::decrementDecoderCount()
{
    return --m_decoderCount;
}

int Dispatcher::bitsPerPixel()
{
    return m_client->format.bitsPerPixel;
}

Uploader &Dispatcher::uploader() const
{
    return m_uploader;
}

void Dispatcher::setFinished(bool finished)
{
    m_finished = finished;
}

bool Dispatcher::finished() const
{
    return m_finished;
}

void Dispatcher::open(QString host, int port)
{
    m_client = rfbGetClient(8, 3, 4);

    m_client->canHandleNewFBSize = FALSE;
    m_client->serverHost = host.toLocal8Bit().data();
    m_client->serverPort = port;
    m_client->MallocFrameBuffer = Dispatcher::MallocFrameBuffer;
    m_client->GotCopyRect = Dispatcher::GotCopyRect;
    m_client->GotFillRect = Dispatcher::GotFillRect;
    m_client->GotBitmap = Dispatcher::GotBitmap;
    m_client->GotJpeg = Dispatcher::GotJpeg;
    m_client->FinishedFrameBufferUpdate = Dispatcher::FinishedFrameBufferUpdate;

    rfbInitClient(m_client, NULL, NULL);
    rfbClientSetClientData(m_client, 0, this);
}
void Dispatcher::refresh()
{
    if(m_client == NULL)
        return;

    m_finished = false;
    int message = WaitForMessage(m_client, 500);
    if(message < 0) {
        emit error();
        return;
    } else if (message) {
        if(!HandleRFBServerMessage(m_client)) {
            emit error();
            return;
        }
    }
}

void Dispatcher::terminate()
{
    if(m_client == NULL)
        return;

    rfbClientCleanup(m_client);
}

void Dispatcher::decodingFinished(const QImage &bitmap, const QPoint &position, const QSize &size)
{
    m_uploader.gotBitmap(bitmap, position, size);
    if(m_finished && decrementDecoderCount() == 0) {
        m_uploader.finishedUpdate();
        emit updateFramebuffer();
    }
}
