#ifndef RFBDISPATCH_H
#define RFBDISPATCH_H

#include <QCoreApplication>
#include <QThreadPool>
#include <QObject>
#include <QPoint>
#include <QSize>
#include <QColor>
#include <rfb/rfbclient.h>

#include "uploader.h"
#include "jpegdecoder.h"

class Dispatcher : public QObject
{
    Q_OBJECT

public:
    // libvnc hooks
    static rfbBool MallocFrameBuffer(rfbClient* client);
    static void GotCopyRect(rfbClient* client, int src_x, int src_y, int w, int h, int dest_x, int dest_y);
    static void GotFillRect(rfbClient* client, int x, int y, int w, int h, uint32_t colour);
    static void GotBitmap(rfbClient* client, const uint8_t* buffer, int x, int y, int w, int h);
    static rfbBool GotJpeg(rfbClient* client, const uint8_t* buffer, int length, int x, int y, int w, int h);
    static void FinishedFrameBufferUpdate(rfbClient* client);


    unsigned int decoderCount() const;
    unsigned int incrementDecoderCount();
    unsigned int decrementDecoderCount();
    int bitsPerPixel();
    Uploader &uploader() const;
    void setFinished(bool finished);

private:
    rfbClient *m_client;
    unsigned int m_decoderCount;
    bool m_finished;
    Uploader m_uploader;
signals:
    void resizeFramebuffer(const QSize &size);
    void updateFramebuffer();
    void error();
public slots:
    void open(QString host, int port);
    void refresh();
    void terminate();
    void decodingFinished(const QImage &bitmap, const QPoint &position, const QSize &size);
};

#endif // RFBDISPATCH_H
