#include "uploader.h"

Uploader::Uploader() : m_fill_buffer(QOpenGLBuffer(QOpenGLBuffer::VertexBuffer))
{
    // "applications must ensure that create() is only called on the main (GUI) thread"
    m_surface.create();
    m_uploader_context.setShareContext(QOpenGLContext::globalShareContext());
    m_uploader_context.create();
    m_uploader_context.makeCurrent(m_surface);

    m_fill_buffer.create();
    m_draw_buffer.create();

    // A single rectangle takes 20 float values to describe (4 vertices * (2 coordinate + 3 color))
    m_fill_data.reserve(100);
    m_draw_data.reserve(100);
}

void Uploader::startedUpdate()
{
    // TODO: bind the framebuffer as texture
}

void Uploader::gotCopyRect(const int &src_x, const int &src_y, const int &w, const int &h, const int &dest_x, const int &dest_y)
{
    // TODO: render parts of the framebuffer to temporary texture
}

void Uploader::gotFillRect(const int x, const int y, const int width, const int height, const float red, const float green, const float blue)
{
    // |---|---|---|---|---|
    // | X | Y | R | G | B |
    // |---|---|---|---|---|
    // NOTE: -coordinates are transformed to normalized device coordinates in the vertex shader
    //       -coordinates and colors are extended to vec4 in the vertex shader
    //       -OpenGL ES 2.0 does not support integer values as input,
    //        thus we need to cast to float on the CPU side :(
    // bottom left
    m_fill_data.append((float) x);
    m_fill_data.append((float) (y + height));
    m_fill_data.append(red);
    m_fill_data.append(green);
    m_fill_data.append(blue);
    // top left
    m_fill_data.append((float) x);
    m_fill_data.append((float) y);
    m_fill_data.append(red);
    m_fill_data.append(green);
    m_fill_data.append(blue);
    // bottom right
    m_fill_data.append((float) (x + width));
    m_fill_data.append((float) (y + height));
    m_fill_data.append(red);
    m_fill_data.append(green);
    m_fill_data.append(blue);
    // top right
    m_fill_data.append((float) (x + width));
    m_fill_data.append((float) y);
    m_fill_data.append(red);
    m_fill_data.append(green);
    m_fill_data.append(blue);
}

void Uploader::gotBitmap(const QImage &image, const int x, const int y, const int width, const int height)
{
    m_draw_textures.append(new QOpenGLTexture(image, QOpenGLTexture::DontGenerateMipMaps));

    // bottom left
    m_draw_data.append((float) x);
    m_draw_data.append((float) (y + height));
    m_draw_data.append(0.0f);
    m_draw_data.append(0.0f);
    // top left
    m_draw_data.append((float) x);
    m_draw_data.append((float) y);
    m_draw_data.append(0.0f);
    m_draw_data.append(1.0f);
    // bottom right
    m_draw_data.append((float) (x + width));
    m_draw_data.append((float) (y + height));
    m_draw_data.append(1.0f);
    m_draw_data.append(0.0f);
    // top right
    m_draw_data.append((float) (x + width));
    m_draw_data.append((float) y);
    m_draw_data.append(1.0f);
    m_draw_data.append(1.0f);
}

void Uploader::finishedUpdate()
{
    QOpenGLFunctions *fun = QOpenGLContext::currentContext()->functions();

    // Upload fill
    int fill_size = m_fill_data.size() * sizeof(float);
    m_fill_buffer.bind();
    m_fill_buffer.allocate(fill_size);
    m_fill_buffer.write(0, m_fill_data.constData(), fill_size);
    m_fill_buffer.release();
    m_fill_data.clear();

    // Upload draw
    int draw_size = m_draw_data.size() * sizeof(float);
    m_draw_buffer.bind();
    m_draw_buffer.allocate(draw_size);
    m_fill_buffer.write(0, m_fill_data.constData(), fill_size);
    m_fill_buffer.release();
    m_fill_data.clear();
}


QOpenGLBuffer Uploader::draw_buffer() const
{
    return m_draw_buffer;
}
QOpenGLBuffer Uploader::fill_buffer() const
{
    return m_fill_buffer;
}
