#include "vncrenderer.h"

VncRenderer::VncRenderer(QOpenGLBuffer fill_buffer, QOpenGLBuffer draw_buffer, QOpenGLTexture draw_texture) :
    m_fill_buffer(fill_buffer), m_draw_buffer(draw_buffer), m_draw_texture(draw_texture)
{
    QOpenGLShader fill_vshader(QOpenGLShader::Vertex, &m_fill_program);
    fill_vshader.compileSourceCode("uniform highp mat4 ortho;\n"
                                   "attribute highp vec2 position;\n"
                                   "attribute lowp vec3 color;\n"
                                   "varying lowp vec4 varcolor;\n"
                                   "void main(void)\n"
                                   "{\n"
                                   "    varcolor = vec4(color, 1.0);\n"
                                   "    gl_Position = ortho * vec4(position, 0.0, 1.0);\n"
                                   "}\n");

    QOpenGLShader fill_fshader(QOpenGLShader::Fragment, &m_fill_program);
    fill_fshader.compileSourceCode("varying lowp vec4 varcolor;\n"
                                   "void main(void)\n"
                                   "{\n"
                                   "    gl_FragColor = varcolor;\n"
                                   "}\n");

    m_fill_program.addShader(&fill_vshader);
    m_fill_program.addShader(&fill_fshader);
    m_fill_program.link();
    m_fill_program.enableAttributeArray("position");
    m_fill_program.enableAttributeArray("color");

    QOpenGLShader draw_vshader(QOpenGLShader::Vertex, &m_fill_program);
    draw_vshader.compileSourceCode("uniform highp mat2 ortho;\n"
                                   "uniform highp mat4 atlasortho;\n"
                                   "attribute highp vec2 position;\n"
                                   "attribute lowp vec2 texture;\n"
                                   "varying lowp vec2 vartexture;\n"
                                   "void main(void)\n"
                                   "{\n"
                                   "    vartexture = atlasortho * texture;\n"
                                   "    gl_Position = ortho * vec4(position, 0.0, 1.0);\n"
                                   "}\n");

    QOpenGLShader draw_fshader(QOpenGLShader::Fragment, &m_fill_program);
    draw_fshader.compileSourceCode("uniform sampler2D atlas;\n"
                                   "varying lowp vec2 vartexture;\n"
                                   "void main(void)\n"
                                   "{\n"
                                   "    gl_FragColor = texture2D(atlas, vartexture);\n"
                                   "}\n");

    m_draw_program.addShader(&draw_vshader);
    m_draw_program.addShader(&draw_fshader);
    m_draw_program.link();
    m_draw_program.enableAttributeArray("position");
    m_draw_program.enableAttributeArray("texture");
}

void VncRenderer::render()
{
    QOpenGLFunctions *fun = QOpenGLContext::currentContext()->functions();

    // fill rectangles with solid color
    m_fill_program.bind();
    m_fill_buffer.bind();
    for(unsigned int i = 0; i < m_fill_count; i++)
        fun->glDrawArrays(GL_TRIANGLES, 0, 4);
    m_fill_buffer.release();
    m_fill_program.release();

    m_draw_program.bind();
    m_draw_buffer.bind();
    m_draw_texture.bind();

    foreach(const Uploader::CopyToAtlas &copy, m_copy_data)
        fun->glTexImage2D(GL_TEXTURE_2D,
                          0,
                          GL_RGBA32,);

    // draw the vertices of the texture atlas
    for(int i = 0; i < m_draw_count; i++)
        fun->glDrawArrays(GL_TRIANGLES, 0, 4);

    m_draw_texture.release();
    m_draw_buffer.release();
    m_draw_program.release();

    QQuickWindow::resetOpenGLState();
    emit rendered();
}

QOpenGLFramebufferObject *VncRenderer::createFramebufferObject(const QSize &size)
{
    // Set the orthogonal transformation matrix
    m_fill_program.bind();
    QMatrix4x4 ortho;
    ortho.ortho(QRect(QPoint(0, 0), size));
    m_fill_program.setUniformValue("ortho", mat);
    m_fill_program.release();

    return new QOpenGLFramebufferObject(size);
}

void VncRenderer::synchronize(QQuickFramebufferObject *object)
{
    QVnc *vnc = static_cast<QVnc*>(object);
}
