#ifndef VNCRENDERER_H
#define VNCRENDERER_H

#include <QObject>
#include <QQuickWindow>
#include <QQuickFramebufferObject>
#include <QOpenGLFramebufferObject>
#include <QOpenGLContext>
#include <QOpenGLFunctions>
#include <QMatrix4x4>
#include <QOpenGLShaderProgram>
#include <QOpenGLBuffer>
#include <QOpenGLTexture>

#include "qvnc.h"
#include "uploader.h"

class VncRenderer : public QQuickFramebufferObject::Renderer
{
    Q_OBJECT
public:
    VncRenderer(QOpenGLBuffer fill_buffer, QOpenGLBuffer draw_buffer, QOpenGLTexture draw_texture);
protected:
    void render() override;
    QOpenGLFramebufferObject *createFramebufferObject(const QSize &size) override;
    void synchronize(QQuickFramebufferObject *) override;
private:
    // for filling a rectangle with a solid color
    QOpenGLShaderProgram m_fill_program;
    QOpenGLBuffer m_fill_buffer;
    unsigned int m_fill_count;
    // for copying portions of the framebuffer
    QVector<Uploader::CopyToAtlas> m_copy_data;
    // for drawing from a texture atlas
    QOpenGLShaderProgram m_draw_program;
    QOpenGLBuffer m_draw_buffer;
    QVector<*QOpenGLTexture> m_draw_texture;
    unsigned int m_draw_count;

    QMatrix4x4 m_ortho;
signals:
    void rendered();
};

#endif // VNCRENDERER_H
